@@ -92,8 +92,20 @@ impl State {
         repo.set_push_remote(repo_config.push_remote());
         repo.set_pull_remote(repo_config.pull_remote());
 
-        let branches = git_stack::git::Branches::new(repo.local_branches());
-        let protected_branches = branches.protected(&protected);
+        let mut branches = git_stack::git::Branches::new([]);
+        let mut protected_branches = git_stack::git::Branches::new([]);
+        for branch in repo.local_branches() {
+            if protected.is_protected(&branch.name) {
+                log::trace!("Branch {} is protected", branch);
+                if let Some(remote) = repo.find_remote_branch(repo.pull_remote(), &branch.name) {
+                    protected_branches.insert(remote.clone());
+                    branches.insert(remote);
+                } else {
+                    protected_branches.insert(branch.clone());
+                }
+            }
+            branches.insert(branch);
+        }
         let head_commit = repo.head_commit();
         let base = args
             .base
@@ -110,27 +122,15 @@ impl State {
 
         let stacks = match (base, onto, repo_config.stack()) {
             (Some(base), Some(onto), git_stack::config::Stack::All) => {
-                vec![StackState {
-                    base,
-                    onto,
-                    branches: branches.all(),
-                }]
+                vec![StackState::new(base, onto, branches.all())]
             }
             (Some(base), None, git_stack::config::Stack::All) => {
-                let onto = base.clone();
-                vec![StackState {
-                    base,
-                    onto,
-                    branches: branches.all(),
-                }]
+                let onto = resolve_onto_from_base(&repo, &base);
+                vec![StackState::new(base, onto, branches.all())]
             }
             (None, Some(onto), git_stack::config::Stack::All) => {
-                let base = onto.clone();
-                vec![StackState {
-                    base,
-                    onto,
-                    branches: branches.all(),
-                }]
+                let base = resolve_base_from_onto(&repo, &onto);
+                vec![StackState::new(base, onto, branches.all())]
             }
             (None, None, git_stack::config::Stack::All) => {
                 let mut stack_branches = std::collections::BTreeMap::new();
@@ -139,37 +139,60 @@ impl State {
                         resolve_implicit_base(&repo, branch_id, &branches, &protected_branches)
                             .with_code(proc_exit::Code::USAGE_ERR)?;
                     stack_branches
-                        .entry(base_branch)
+                        .entry(AnnotatedOid::with_branch(base_branch))
                         .or_insert_with(git_stack::git::Branches::default)
                         .extend(branch.iter().cloned());
                 }
                 stack_branches
                     .into_iter()
-                    .map(|(base, branches)| {
-                        let onto = base.clone();
-                        StackState {
-                            base,
-                            onto,
-                            branches,
-                        }
+                    .map(|(onto, branches)| {
+                        let base = resolve_base_from_onto(&repo, &onto);
+                        StackState::new(base, onto, branches)
                     })
                     .collect()
             }
             (base, onto, stack) => {
-                let base = base
-                    .map(Result::Ok)
-                    .unwrap_or_else(|| {
-                        resolve_implicit_base(&repo, head_commit.id, &branches, &protected_branches)
-                    })
-                    .with_code(proc_exit::Code::USAGE_ERR)?;
-                let onto = onto.unwrap_or_else(|| base.clone());
+                let (base, onto) = match (base, onto) {
+                    (Some(base), Some(onto)) => (base, onto),
+                    (Some(base), None) => {
+                        let onto = resolve_onto_from_base(&repo, &base);
+                        (base, onto)
+                    }
+                    (None, Some(onto)) => {
+                        let base = AnnotatedOid::with_branch(
+                            resolve_implicit_base(
+                                &repo,
+                                head_commit.id,
+                                &branches,
+                                &protected_branches,
+                            )
+                            .with_code(proc_exit::Code::USAGE_ERR)?,
+                        );
+                        // HACK: Since `base` might have come back with a remote branch, treat it as an
+                        // "onto" to find the local version.
+                        let base = resolve_base_from_onto(&repo, &base);
+                        (base, onto)
+                    }
+                    (None, None) => {
+                        let onto = resolve_implicit_base(
+                            &repo,
+                            head_commit.id,
+                            &branches,
+                            &protected_branches,
+                        )
+                        .map(AnnotatedOid::with_branch)
+                        .with_code(proc_exit::Code::USAGE_ERR)?;
+                        let base = resolve_base_from_onto(&repo, &onto);
+                        (base, onto)
+                    }
+                };
                 let merge_base_oid = repo
                     .merge_base(base.id, head_commit.id)
                     .ok_or_else(|| {
                         git2::Error::new(
                             git2::ErrorCode::NotFound,
                             git2::ErrorClass::Reference,
-                            format!("could not find base between {} and HEAD", base.name),
+                            format!("could not find base between {} and HEAD", base),
                         )
                     })
                     .with_code(proc_exit::Code::USAGE_ERR)?;
@@ -185,14 +208,19 @@ impl State {
                     }
                     git_stack::config::Stack::All => unreachable!("Covered in another branch"),
                 };
-                vec![StackState {
-                    base,
-                    onto,
-                    branches: stack_branches,
-                }]
+                vec![StackState::new(base, onto, stack_branches)]
             }
         };
 
+        for stack in &stacks {
+            if let Some(branch) = stack.base.branch.clone() {
+                protected_branches.insert(branch);
+            }
+            if let Some(branch) = stack.onto.branch.clone() {
+                protected_branches.insert(branch);
+            }
+        }
+
         Ok(Self {
             repo,
             branches,
@@ -232,32 +260,31 @@ impl State {
 
 #[derive(Debug)]
 struct StackState {
-    base: git_stack::git::Branch,
-    onto: git_stack::git::Branch,
+    base: AnnotatedOid,
+    onto: AnnotatedOid,
     branches: git_stack::git::Branches,
 }
 
 impl StackState {
-    fn update(&mut self, repo: &dyn git_stack::git::Repo) -> eyre::Result<()> {
-        self.base = repo
-            .find_local_branch(self.base.name.as_str())
-            .ok_or_else(|| eyre::eyre!("can no longer find branch {}", self.base.name))?;
-        self.onto = repo
-            .find_local_branch(self.onto.name.as_str())
-            .ok_or_else(|| eyre::eyre!("can no longer find branch {}", self.onto.name))?;
-        self.branches.update(repo);
-        Ok(())
+    fn new(base: AnnotatedOid, onto: AnnotatedOid, mut branches: git_stack::git::Branches) -> Self {
+        if let Some(base) = &base.branch {
+            branches.insert(base.clone());
+        }
+        if let Some(onto) = &onto.branch {
+            branches.insert(onto.clone());
+        }
+        Self {
+            base,
+            onto,
+            branches,
+        }
     }
 
-    fn graphed_branches(&self) -> git_stack::git::Branches {
-        let mut graphed_branches = self.branches.clone();
-        if !graphed_branches.contains_oid(self.base.id) {
-            graphed_branches.insert(self.base.clone());
-        }
-        if !graphed_branches.contains_oid(self.onto.id) {
-            graphed_branches.insert(self.onto.clone());
-        }
-        graphed_branches
+    fn update(&mut self, repo: &dyn git_stack::git::Repo) -> eyre::Result<()> {
+        self.base.update(repo)?;
+        self.onto.update(repo)?;
+        self.branches.update(repo);
+        Ok(())
     }
 }
 
@@ -280,7 +307,7 @@ pub fn stack(
             .flat_map(|stack| stack.branches.iter())
             .filter(|(oid, _)| !state.protected_branches.contains_oid(*oid))
             .flat_map(|(_, b)| b.iter())
-            .filter_map(|b| b.push_id.map(|_| b.name.as_str()))
+            .filter_map(|b| b.push_id.and_then(|_| b.local_name()))
             .collect();
         push_branches.sort_unstable();
         if !push_branches.is_empty() {
@@ -293,18 +320,17 @@ pub fn stack(
         }
 
         for stack in state.stacks.iter() {
-            if state.protected_branches.contains_oid(stack.onto.id) {
-                match git_fetch_upstream(&mut state.repo, stack.onto.name.as_str()) {
-                    Ok(_) => (),
-                    Err(err) => {
-                        log::warn!("Skipping pull of `{}`, {}", stack.onto.name, err);
+            if let Some(branch) = &stack.onto.branch {
+                if let Some(remote) = &branch.remote {
+                    match git_fetch_upstream(remote, branch.name.as_str()) {
+                        Ok(_) => (),
+                        Err(err) => {
+                            log::warn!("Skipping pull of `{}`, {}", branch, err);
+                        }
                     }
+                } else {
+                    log::warn!("Skipping pull of `{}` local branch", branch);
                 }
-            } else {
-                log::warn!(
-                    "Skipping pull of `{}`, not a protected branch",
-                    stack.onto.name
-                );
             }
         }
         state.update().with_code(proc_exit::Code::FAILURE)?;
@@ -355,7 +381,17 @@ pub fn stack(
             .map(|stack| {
                 let script = plan_changes(&state, stack).with_code(proc_exit::Code::FAILURE)?;
                 if script.is_branch_deleted(&head_branch) {
-                    head_branch = stack.onto.name.clone();
+                    // Current branch is deleted, fallback to the local version of the onto branch,
+                    // if possible.
+                    if let Some(local_name) = stack
+                        .onto
+                        .branch
+                        .as_ref()
+                        .map(|b| b.name.as_str())
+                        .filter(|n| state.repo.find_local_branch(n).is_some())
+                    {
+                        head_branch = local_name.to_owned();
+                    }
                 }
                 Ok(script)
             })
@@ -411,26 +447,15 @@ pub fn stack(
 }
 
 fn plan_changes(state: &State, stack: &StackState) -> eyre::Result<git_stack::git::Script> {
-    log::trace!("Planning stack changes with base={}", stack.base.name,);
-    let graphed_branches = stack.graphed_branches();
+    log::trace!("Planning stack changes with base={}", stack.base,);
+    let graphed_branches = stack.branches.clone();
     let base_commit = state
         .repo
         .find_commit(stack.base.id)
         .expect("base branch is valid");
     let mut graph = git_stack::graph::Graph::from_branches(&state.repo, graphed_branches)?;
     graph.insert(&state.repo, git_stack::graph::Node::new(base_commit))?;
-    for branch in state.protected_branches.iter().flat_map(|(_, b)| b) {
-        if let Some(pull_id) = branch.pull_id {
-            let pull_commit = state
-                .repo
-                .find_commit(pull_id)
-                .expect("base branch is valid");
-            graph.insert(&state.repo, git_stack::graph::Node::new(pull_commit))?;
-        }
-    }
     git_stack::graph::protect_branches(&mut graph, &state.repo, &state.protected_branches);
-    let bases = git_stack::git::Branches::new([stack.base.clone(), stack.onto.clone()]);
-    git_stack::graph::protect_branches(&mut graph, &state.repo, &bases);
     if let Some(protect_commit_count) = state.protect_commit_count {
         git_stack::graph::protect_large_branches(&mut graph, protect_commit_count);
     }
@@ -445,9 +470,9 @@ fn plan_changes(state: &State, stack: &StackState) -> eyre::Result<git_stack::gi
 
     let mut dropped_branches = Vec::new();
     if state.rebase {
-        log::trace!("Rebasing onto {}", stack.onto.name);
-        let onto_id = stack.onto.pull_id.unwrap_or(stack.onto.id);
-        let pull_start_id = stack.onto.id;
+        log::trace!("Rebasing onto {}", stack.onto);
+        let onto_id = stack.onto.id;
+        let pull_start_id = stack.base.id;
         let pull_start_id = state
             .repo
             .merge_base(pull_start_id, onto_id)
@@ -490,7 +515,7 @@ fn plan_changes(state: &State, stack: &StackState) -> eyre::Result<git_stack::gi
 fn push(state: &mut State) -> eyre::Result<()> {
     let mut graphed_branches = git_stack::git::Branches::new(None.into_iter());
     for stack in state.stacks.iter() {
-        let stack_graphed_branches = stack.graphed_branches();
+        let stack_graphed_branches = stack.branches.clone();
         graphed_branches.extend(stack_graphed_branches.into_iter().flat_map(|(_, b)| b));
     }
     let mut graph = git_stack::graph::Graph::from_branches(&state.repo, graphed_branches)?;
@@ -538,36 +563,23 @@ fn show(state: &State, colored_stdout: bool, colored_stderr: bool) -> eyre::Resu
 
     let mut graphs = Vec::with_capacity(state.stacks.len());
     for stack in state.stacks.iter() {
-        let graphed_branches = stack.graphed_branches();
+        let graphed_branches = stack.branches.clone();
         if graphed_branches.len() == 1 && abbrev_graph {
             let branches = graphed_branches.iter().next().unwrap().1;
             if branches.len() == 1 && branches[0].id != state.head_commit.id {
-                empty_stacks.push(format!("{}", palette_stderr.info.paint(&branches[0].name)));
+                empty_stacks.push(format!("{}", palette_stderr.info.paint(&branches[0])));
                 continue;
             }
         }
 
-        log::trace!("Rendering stack base={}", stack.base.name,);
+        log::trace!("Rendering stack base={}", stack.base,);
         let base_commit = state
             .repo
             .find_commit(stack.base.id)
             .expect("base branch is valid");
         let mut graph = git_stack::graph::Graph::from_branches(&state.repo, graphed_branches)?;
         graph.insert(&state.repo, git_stack::graph::Node::new(base_commit))?;
-        for branch in state.protected_branches.iter().flat_map(|(_, b)| b) {
-            if let Some(pull_id) = branch.pull_id {
-                if state.repo.merge_base(pull_id, graph.root_id()) == Some(graph.root_id()) {
-                    let pull_commit = state
-                        .repo
-                        .find_commit(pull_id)
-                        .expect("base branch is valid");
-                    graph.insert(&state.repo, git_stack::graph::Node::new(pull_commit))?;
-                }
-            }
-        }
         git_stack::graph::protect_branches(&mut graph, &state.repo, &state.protected_branches);
-        let bases = git_stack::git::Branches::new([stack.base.clone(), stack.onto.clone()]);
-        git_stack::graph::protect_branches(&mut graph, &state.repo, &bases);
         if let Some(protect_commit_count) = state.protect_commit_count {
             let protected =
                 git_stack::graph::protect_large_branches(&mut graph, protect_commit_count);
@@ -605,9 +617,9 @@ fn show(state: &State, colored_stdout: bool, colored_stderr: bool) -> eyre::Resu
         if state.dry_run {
             // Show as-if we performed all mutations
             if state.rebase {
-                log::trace!("Rebasing onto {}", stack.onto.name);
-                let onto_id = stack.onto.pull_id.unwrap_or(stack.onto.id);
-                let pull_start_id = stack.onto.id;
+                log::trace!("Rebasing onto {}", stack.onto);
+                let onto_id = stack.onto.id;
+                let pull_start_id = stack.base.id;
                 let pull_start_id = state
                     .repo
                     .merge_base(pull_start_id, onto_id)
@@ -708,12 +720,72 @@ fn show(state: &State, colored_stdout: bool, colored_stderr: bool) -> eyre::Resu
     Ok(())
 }
 
-fn resolve_explicit_base(
-    repo: &dyn git_stack::git::Repo,
-    base: &str,
-) -> eyre::Result<git_stack::git::Branch> {
-    repo.find_local_branch(base)
-        .ok_or_else(|| eyre::eyre!("could not find branch {:?}", base))
+#[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
+struct AnnotatedOid {
+    id: git2::Oid,
+    branch: Option<git_stack::git::Branch>,
+}
+
+impl AnnotatedOid {
+    fn new(id: git2::Oid) -> Self {
+        Self { id, branch: None }
+    }
+
+    fn with_branch(branch: git_stack::git::Branch) -> Self {
+        Self {
+            id: branch.id,
+            branch: Some(branch),
+        }
+    }
+
+    fn update(&mut self, repo: &dyn git_stack::git::Repo) -> eyre::Result<()> {
+        let branch = self.branch.as_ref().and_then(|branch| {
+            if let Some(remote) = &branch.remote {
+                repo.find_remote_branch(remote, &branch.name)
+            } else {
+                repo.find_local_branch(&branch.name)
+            }
+        });
+        if let Some(branch) = branch {
+            *self = Self::with_branch(branch);
+        }
+        Ok(())
+    }
+}
+
+impl std::fmt::Display for AnnotatedOid {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        if let Some(branch) = &self.branch {
+            branch.fmt(f)
+        } else {
+            self.id.fmt(f)
+        }
+    }
+}
+
+fn resolve_explicit_base(repo: &git_stack::git::GitRepo, base: &str) -> eyre::Result<AnnotatedOid> {
+    let (obj, r) = repo.raw().revparse_ext(base)?;
+    if let Some(r) = r {
+        if r.is_tag() {
+            return Ok(AnnotatedOid::new(obj.id()));
+        }
+
+        let branch = if r.is_remote() {
+            let (remote, name) = r
+                .shorthand()
+                .ok_or_else(|| eyre::eyre!("Expected branch, got `{}`", base))?
+                .split_once('/')
+                .unwrap();
+            repo.find_remote_branch(remote, name)
+                .ok_or_else(|| eyre::eyre!("Could not find branch {:?}", r.shorthand()))
+        } else {
+            repo.find_local_branch(base)
+                .ok_or_else(|| eyre::eyre!("Could not find branch {:?}", base))
+        }?;
+        Ok(AnnotatedOid::with_branch(branch))
+    } else {
+        Ok(AnnotatedOid::new(obj.id()))
+    }
 }
 
 fn resolve_implicit_base(
@@ -723,13 +795,13 @@ fn resolve_implicit_base(
     protected_branches: &git_stack::git::Branches,
 ) -> eyre::Result<git_stack::git::Branch> {
     let branch = git_stack::git::find_protected_base(repo, protected_branches, head_oid)
-        .ok_or_else(|| eyre::eyre!("could not find a protected branch to use as a base"))?;
+        .ok_or_else(|| eyre::eyre!("Could not find a protected branch to use as a base"))?;
     log::debug!(
         "Chose branch {} as the base for {}",
-        branch.name,
+        branch,
         branches
             .get(head_oid)
-            .map(|b| b[0].name.clone())
+            .map(|b| b[0].to_string())
             .or_else(|| {
                 repo.find_commit(head_oid)?
                     .summary
@@ -742,6 +814,26 @@ fn resolve_implicit_base(
     Ok(branch.clone())
 }
 
+fn resolve_base_from_onto(repo: &git_stack::git::GitRepo, onto: &AnnotatedOid) -> AnnotatedOid {
+    // HACK: Assuming the local branch is the current base for all the commits
+    onto.branch
+        .as_ref()
+        .filter(|b| b.remote.is_some())
+        .and_then(|b| repo.find_local_branch(&b.name))
+        .map(AnnotatedOid::with_branch)
+        .unwrap_or_else(|| onto.clone())
+}
+
+fn resolve_onto_from_base(repo: &git_stack::git::GitRepo, base: &AnnotatedOid) -> AnnotatedOid {
+    // HACK: Assuming the local branch is the current base for all the commits
+    base.branch
+        .as_ref()
+        .filter(|b| b.remote.is_none())
+        .and_then(|b| repo.find_remote_branch(repo.pull_remote(), &b.name))
+        .map(AnnotatedOid::with_branch)
+        .unwrap_or_else(|| base.clone())
+}
+
 fn git_prune_development(
     repo: &mut git_stack::git::GitRepo,
     branches: &[&str],
@@ -788,8 +880,7 @@ fn git_prune_development(
     Ok(())
 }
 
-fn git_fetch_upstream(repo: &mut git_stack::git::GitRepo, branch_name: &str) -> eyre::Result<()> {
-    let remote = repo.pull_remote();
+fn git_fetch_upstream(remote: &str, branch_name: &str) -> eyre::Result<()> {
     log::debug!("git fetch {} {}", remote, branch_name);
     // A little uncertain about some of the weirder authentication needs, just deferring to `git`
     // instead of using `libgit2`
@@ -828,7 +919,7 @@ fn git_push(
     if failed.is_empty() {
         Ok(())
     } else {
-        eyre::bail!("could not push {}", failed.into_iter().join(", "));
+        eyre::bail!("Could not push {}", failed.into_iter().join(", "));
     }
 }
 
@@ -839,10 +930,16 @@ fn git_push_node(
 ) -> Vec<String> {
     let mut failed = Vec::new();
     for branch in node.branches.iter() {
+        let local_branch = if let Some(local_name) = branch.local_name() {
+            local_name
+        } else {
+            continue;
+        };
+
         if node.pushable {
             let raw_branch = repo
                 .raw()
-                .find_branch(&branch.name, git2::BranchType::Local)
+                .find_branch(local_branch, git2::BranchType::Local)
                 .expect("all referenced branches exist");
             let upstream_set = raw_branch.upstream().is_ok();
 
@@ -852,26 +949,26 @@ fn git_push_node(
                 args.push("--set-upstream");
             }
             args.push(remote);
-            args.push(&branch.name);
+            args.push(local_branch);
             log::trace!("git {}", args.join(" "),);
             if !dry_run {
                 let status = std::process::Command::new("git").args(&args).status();
                 match status {
                     Ok(status) => {
                         if !status.success() {
-                            failed.push(branch.name.clone());
+                            failed.push(local_branch.to_owned());
                         }
                     }
                     Err(err) => {
                         log::debug!("`git push` failed with {}", err);
-                        failed.push(branch.name.clone());
+                        failed.push(local_branch.to_owned());
                     }
                 }
             }
         } else if node.action.is_protected() {
-            log::debug!("Skipping push of `{}`, protected", branch.name);
+            log::debug!("Skipping push of `{}`, protected", branch);
         } else {
-            log::debug!("Skipping push of `{}`", branch.name);
+            log::debug!("Skipping push of `{}`", branch);
         }
     }
 
@@ -891,8 +988,8 @@ fn list(
         let mut branches: Vec<_> = node.branches.iter().collect();
         branches.sort();
         for b in branches {
-            if protected.into_iter().flatten().contains(&b) {
-                // Base / protected branches are just show for context, they aren't part of the
+            if b.remote.is_some() || protected.into_iter().flatten().contains(&b) {
+                // Base, remote, and protected branches are just shown for context, they aren't part of the
                 // stack, so skip them here
                 continue;
             }
@@ -1177,6 +1274,9 @@ impl<'r> Tree<'r> {
                 }
                 tree.push(stack_tree);
             } else {
+                if i != 0 && !stack.is_empty() {
+                    tree.push(termtree::Tree::root(joint).with_glyphs(SPACE_GLYPHS));
+                }
                 for child_tree in stack.into_iter() {
                     let child = RenderNode {
                         repo,
@@ -1261,6 +1361,14 @@ const GLYPHS: termtree::GlyphPalette = termtree::GlyphPalette {
     ..termtree::GlyphPalette::new()
 };
 
+const SPACE_GLYPHS: termtree::GlyphPalette = termtree::GlyphPalette {
+    middle_item: "│",
+    last_item: " ",
+    item_indent: " ",
+    skip_indent: " ",
+    ..termtree::GlyphPalette::new()
+};
+
 const JOINT_GLYPHS: termtree::GlyphPalette = termtree::GlyphPalette {
     item_indent: "─┐",
     ..termtree::GlyphPalette::new()
@@ -1294,15 +1402,27 @@ impl<'r> std::fmt::Display for RenderNode<'r> {
             } else {
                 let mut branches: Vec<_> = node.branches.iter().collect();
                 branches.sort_by_key(|b| {
-                    let is_head = self.head_branch.id == b.id && self.head_branch.name == b.name;
+                    let is_head = self.head_branch.id == b.id
+                        && self.head_branch.remote == b.remote
+                        && self.head_branch.name == b.name;
                     let head_first = !is_head;
-                    (head_first, &b.name)
+                    (head_first, &b.remote, &b.name)
                 });
                 write!(
                     f,
                     "{}",
                     branches
-                        .into_iter()
+                        .iter()
+                        .filter(|b| {
+                            if b.remote.is_some() {
+                                let local_present = branches
+                                    .iter()
+                                    .any(|b| b.local_name() == Some(b.name.as_str()));
+                                !local_present
+                            } else {
+                                true
+                            }
+                        })
                         .map(|b| {
                             format!(
                                 "{}{}",
@@ -1350,17 +1470,22 @@ fn format_branch_name<'d>(
     protected_branches: &'d git_stack::git::Branches,
     palette: &'d Palette,
 ) -> impl std::fmt::Display + 'd {
-    if head_branch.id == branch.id && head_branch.name == branch.name {
-        palette.highlight.paint(branch.name.as_str())
+    if head_branch.id == branch.id
+        && head_branch.remote == branch.remote
+        && head_branch.name == branch.name
+    {
+        palette.highlight.paint(branch.to_string())
     } else {
         let protected = protected_branches.get(branch.id);
         if protected.into_iter().flatten().contains(&branch) {
-            palette.info.paint(branch.name.as_str())
+            palette.info.paint(branch.to_string())
+        } else if branch.remote.is_some() {
+            palette.info.paint(branch.to_string())
         } else if node.action.is_protected() {
             // Either haven't started dev or it got merged
-            palette.warn.paint(branch.name.as_str())
+            palette.warn.paint(branch.to_string())
         } else {
-            palette.good.paint(branch.name.as_str())
+            palette.good.paint(branch.to_string())
         }
     }
 }
@@ -1373,25 +1498,10 @@ fn format_branch_status<'d>(
 ) -> String {
     // See format_commit_status
     if node.action.is_protected() {
-        match commit_relation(repo, branch.id, branch.pull_id) {
-            Some((0, 0)) => String::new(),
-            Some((local, 0)) => {
-                format!(" {}", palette.warn.paint(format!("({} ahead)", local)))
-            }
-            Some((0, remote)) => {
-                format!(" {}", palette.warn.paint(format!("({} behind)", remote)))
-            }
-            Some((local, remote)) => {
-                format!(
-                    " {}",
-                    palette
-                        .warn
-                        .paint(format!("({} ahead, {} behind)", local, remote)),
-                )
-            }
-            None => {
-                format!(" {}", palette.warn.paint("(no remote)"))
-            }
+        if branch.pull_id.is_none() {
+            format!(" {}", palette.warn.paint("(no remote)"))
+        } else {
+            String::new()
         }
     } else if node.action.is_delete() {
         String::new()
