@@ -2,6 +2,7 @@ use bstr::ByteSlice;
 use itertools::Itertools;
 
 pub trait Repo {
+    fn path(&self) -> Option<&std::path::Path>;
     fn user(&self) -> Option<std::rc::Rc<str>>;
 
     fn is_dirty(&self) -> bool;
@@ -36,19 +37,38 @@ pub trait Repo {
     fn branch(&mut self, name: &str, id: git2::Oid) -> Result<(), git2::Error>;
     fn delete_branch(&mut self, name: &str) -> Result<(), git2::Error>;
     fn find_local_branch(&self, name: &str) -> Option<Branch>;
+    fn find_remote_branch(&self, remote: &str, name: &str) -> Option<Branch>;
     fn local_branches(&self) -> Box<dyn Iterator<Item = Branch> + '_>;
+    fn remote_branches(&self) -> Box<dyn Iterator<Item = Branch> + '_>;
     fn detach(&mut self) -> Result<(), git2::Error>;
     fn switch(&mut self, name: &str) -> Result<(), git2::Error>;
 }
 
 #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct Branch {
+    pub remote: Option<String>,
     pub name: String,
     pub id: git2::Oid,
     pub push_id: Option<git2::Oid>,
     pub pull_id: Option<git2::Oid>,
 }
 
+impl Branch {
+    pub fn local_name(&self) -> Option<&str> {
+        self.remote.is_none().then(|| self.name.as_str())
+    }
+}
+
+impl std::fmt::Display for Branch {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        if let Some(remote) = self.remote.as_deref() {
+            write!(f, "{}/{}", remote, self.name.as_str())
+        } else {
+            write!(f, "{}", self.name.as_str())
+        }
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct Commit {
     pub id: git2::Oid,
@@ -235,6 +255,7 @@ impl GitRepo {
             .and_then(|b| b.get().target());
 
         Some(Branch {
+            remote: None,
             name: name.to_owned(),
             id,
             push_id,
@@ -418,6 +439,62 @@ impl GitRepo {
 
     pub fn find_local_branch(&self, name: &str) -> Option<Branch> {
         let branch = self.repo.find_branch(name, git2::BranchType::Local).ok()?;
+        self.load_local_branch(&branch, name).ok()
+    }
+
+    pub fn find_remote_branch(&self, remote: &str, name: &str) -> Option<Branch> {
+        let qualified = format!("{}/{}", remote, name);
+        let branch = self
+            .repo
+            .find_branch(&qualified, git2::BranchType::Remote)
+            .ok()?;
+        self.load_remote_branch(&branch, remote, name).ok()
+    }
+
+    pub fn local_branches(&self) -> impl Iterator<Item = Branch> + '_ {
+        log::trace!("Loading local branches");
+        self.repo
+            .branches(Some(git2::BranchType::Local))
+            .into_iter()
+            .flatten()
+            .flat_map(move |branch| {
+                let (branch, _) = branch.ok()?;
+                let name = if let Some(name) = branch.name().ok().flatten() {
+                    name
+                } else {
+                    log::debug!(
+                        "Ignoring non-UTF8 branch {:?}",
+                        branch.name_bytes().unwrap().as_bstr()
+                    );
+                    return None;
+                };
+                self.load_local_branch(&branch, name).ok()
+            })
+    }
+
+    pub fn remote_branches(&self) -> impl Iterator<Item = Branch> + '_ {
+        log::trace!("Loading remote branches");
+        self.repo
+            .branches(Some(git2::BranchType::Remote))
+            .into_iter()
+            .flatten()
+            .flat_map(move |branch| {
+                let (branch, _) = branch.ok()?;
+                let name = if let Some(name) = branch.name().ok().flatten() {
+                    name
+                } else {
+                    log::debug!(
+                        "Ignoring non-UTF8 branch {:?}",
+                        branch.name_bytes().unwrap().as_bstr()
+                    );
+                    return None;
+                };
+                let (remote, name) = name.split_once('/').unwrap();
+                self.load_remote_branch(&branch, remote, name).ok()
+            })
+    }
+
+    fn load_local_branch(&self, branch: &git2::Branch, name: &str) -> Result<Branch, git2::Error> {
         let id = branch.get().target().unwrap();
 
         let push_id = self
@@ -437,7 +514,8 @@ impl GitRepo {
             .ok()
             .and_then(|b| b.get().target());
 
-        Some(Branch {
+        Ok(Branch {
+            remote: None,
             name: name.to_owned(),
             id,
             push_id,
@@ -445,49 +523,24 @@ impl GitRepo {
         })
     }
 
-    pub fn local_branches(&self) -> impl Iterator<Item = Branch> + '_ {
-        log::trace!("Loading branches");
-        self.repo
-            .branches(Some(git2::BranchType::Local))
-            .into_iter()
-            .flatten()
-            .flat_map(move |branch| {
-                let (branch, _) = branch.ok()?;
-                let name = if let Some(name) = branch.name().ok().flatten() {
-                    name
-                } else {
-                    log::debug!(
-                        "Ignoring non-UTF8 branch {:?}",
-                        branch.name_bytes().unwrap().as_bstr()
-                    );
-                    return None;
-                };
-                let id = branch.get().target().unwrap();
+    fn load_remote_branch(
+        &self,
+        branch: &git2::Branch,
+        remote: &str,
+        name: &str,
+    ) -> Result<Branch, git2::Error> {
+        let id = branch.get().target().unwrap();
 
-                let push_id = self
-                    .repo
-                    .find_branch(
-                        &format!("{}/{}", self.push_remote(), name),
-                        git2::BranchType::Remote,
-                    )
-                    .ok()
-                    .and_then(|b| b.get().target());
-                let pull_id = self
-                    .repo
-                    .find_branch(
-                        &format!("{}/{}", self.pull_remote(), name),
-                        git2::BranchType::Remote,
-                    )
-                    .ok()
-                    .and_then(|b| b.get().target());
+        let push_id = (remote == self.push_remote()).then(|| id);
+        let pull_id = (remote == self.pull_remote()).then(|| id);
 
-                Some(Branch {
-                    name: name.to_owned(),
-                    id,
-                    push_id,
-                    pull_id,
-                })
-            })
+        Ok(Branch {
+            remote: Some(remote.to_owned()),
+            name: name.to_owned(),
+            id,
+            push_id,
+            pull_id,
+        })
     }
 
     pub fn detach(&mut self) -> Result<(), git2::Error> {
@@ -536,6 +589,9 @@ impl std::fmt::Debug for GitRepo {
 }
 
 impl Repo for GitRepo {
+    fn path(&self) -> Option<&std::path::Path> {
+        Some(self.repo.path())
+    }
     fn user(&self) -> Option<std::rc::Rc<str>> {
         self.user()
     }
@@ -620,10 +676,18 @@ impl Repo for GitRepo {
         self.find_local_branch(name)
     }
 
+    fn find_remote_branch(&self, remote: &str, name: &str) -> Option<Branch> {
+        self.find_remote_branch(remote, name)
+    }
+
     fn local_branches(&self) -> Box<dyn Iterator<Item = Branch> + '_> {
         Box::new(self.local_branches())
     }
 
+    fn remote_branches(&self) -> Box<dyn Iterator<Item = Branch> + '_> {
+        Box::new(self.remote_branches())
+    }
+
     fn detach(&mut self) -> Result<(), git2::Error> {
         self.detach()
     }
@@ -870,6 +934,7 @@ impl InMemoryRepo {
         self.branches.insert(
             name.to_owned(),
             Branch {
+                remote: None,
                 name: name.to_owned(),
                 id,
                 push_id: None,
@@ -893,10 +958,18 @@ impl InMemoryRepo {
         self.branches.get(name).cloned()
     }
 
+    pub fn find_remote_branch(&self, _remote: &str, _name: &str) -> Option<Branch> {
+        None
+    }
+
     pub fn local_branches(&self) -> impl Iterator<Item = Branch> + '_ {
         self.branches.values().cloned()
     }
 
+    pub fn remote_branches(&self) -> impl Iterator<Item = Branch> + '_ {
+        None.into_iter()
+    }
+
     pub fn detach(&mut self) -> Result<(), git2::Error> {
         Ok(())
     }
@@ -940,6 +1013,9 @@ impl<'c> Iterator for CommitsFrom<'c> {
 }
 
 impl Repo for InMemoryRepo {
+    fn path(&self) -> Option<&std::path::Path> {
+        None
+    }
     fn user(&self) -> Option<std::rc::Rc<str>> {
         self.user()
     }
@@ -1024,10 +1100,18 @@ impl Repo for InMemoryRepo {
         self.find_local_branch(name)
     }
 
+    fn find_remote_branch(&self, remote: &str, name: &str) -> Option<Branch> {
+        self.find_remote_branch(remote, name)
+    }
+
     fn local_branches(&self) -> Box<dyn Iterator<Item = Branch> + '_> {
         Box::new(self.local_branches())
     }
 
+    fn remote_branches(&self) -> Box<dyn Iterator<Item = Branch> + '_> {
+        Box::new(self.remote_branches())
+    }
+
     fn detach(&mut self) -> Result<(), git2::Error> {
         self.detach()
     }
